#include "Shader.hpp"

#include <iostream>
#include <fstream>
#include <memory>

using namespace acidInterstate;
using namespace std;



Shader::Shader( const string& vertexShader, const string& geometryShader, const string& fragmentShader ) : programHandle( 0 ), vertexHandle( 0 ), fragmentHandle( 0 ), geometryHandle( 0 )
{
	programHandle = glCreateProgram();

	if ( programHandle == 0 )
	{
		cerr << "Failed to create shader program" << endl;
		system( "PAUSE" );
		exit( EXIT_FAILURE );
	}

	loadShader(vertexShader, GL_VERTEX_SHADER, vertexHandle);
	loadShader(fragmentShader, GL_FRAGMENT_SHADER, fragmentHandle);
	loadShader(geometryShader, GL_GEOMETRY_SHADER, geometryHandle);


	//link();
}

Shader::Shader(const string& vertexShader, const string& fragmentShader) : programHandle(0), vertexHandle(0), fragmentHandle(0), geometryHandle(0)
{
	programHandle = glCreateProgram();

	if (programHandle == 0)
	{
		cerr << "Failed to create shader program" << endl;
		system("PAUSE");
		exit(EXIT_FAILURE);
	}

	loadShader(vertexShader, GL_VERTEX_SHADER, vertexHandle);
	loadShader(fragmentShader, GL_FRAGMENT_SHADER, fragmentHandle);

	link();
}


Shader::~Shader()
{
	glDeleteProgram( programHandle );
	glDeleteShader( vertexHandle );
	glDeleteShader( fragmentHandle );
	glDeleteShader( geometryHandle );
}

void Shader::useShader() const
{
	glUseProgram( programHandle );
}

void Shader::loadShader( const string& shader, GLenum shaderType, GLuint& handle )
{
	ifstream shaderFile( shader );

	if ( shaderFile.good() )
	{
		// (1) read file into string
		string code = string( istreambuf_iterator<char>( shaderFile ), istreambuf_iterator<char>() );
		shaderFile.close();

		// (2) generate shader handle
		handle = glCreateShader( shaderType );
		if ( handle == 0 )
		{
			cerr << "Failed to create shader" << endl;
			system( "PAUSE" );
			exit( EXIT_FAILURE );
		}

		// (3) compile shader
		auto codePtr = code.c_str();
		glShaderSource( handle, 1, &codePtr, nullptr );
		glCompileShader( handle );

		// Testing for compilation errors
		GLint succeeded;
		glGetShaderiv( handle, GL_COMPILE_STATUS, &succeeded );

		if ( succeeded == GL_FALSE || !glIsShader( handle ) )
		{
			GLint logSize;
			glGetShaderiv( handle, GL_INFO_LOG_LENGTH, &logSize );

			auto message = make_unique<char[]>( logSize );
			glGetShaderInfoLog( handle, logSize, nullptr, message.get() );
			cerr << "Failed to compile shader" << endl;
			cerr << message.get() << endl;
			system( "PAUSE" );
			message.reset( nullptr );
			exit( EXIT_FAILURE );
		}
		attachShader(handle);

	}
	else
	{
		cout << "Failed to open shader file: " << shader << endl;
		system( "PAUSE" );
		exit( -1 );
	}
}



void Shader::link()
{

	/*if (vertexHandle != 0)
		glAttachShader(programHandle, vertexHandle);
	if (fragmentHandle != 0)
		glAttachShader(programHandle, fragmentHandle);
	if (geometryHandle != 0)
		glAttachShader(programHandle, geometryHandle);
		*/
	glBindFragDataLocation(programHandle, 0, "fragColor");
	glLinkProgram( programHandle );
}


void Shader::attachShader(GLuint& handle)
{
	glAttachShader(programHandle, handle);
}

