#version 330 core

in vec3 fragNormal;
in vec3 fragPos;
in vec2 fragUV; 

out vec4 color;

uniform sampler2D texture_diffuse1;
uniform vec3 objectColor;

uniform vec3 lightColor;
uniform vec3 lightPositionPoint;
uniform vec3 lightPositionSpot;

uniform vec3 viewPosition;
uniform float cutOff;

const int numberOfPointLights = 8;
uniform vec3 lightPositionPoint1;
uniform vec3 lightPositionPoint2;
uniform vec3 lightPositionPoint3;
uniform vec3 lightPositionPoint4;
uniform vec3 lightPositionPoint5;
uniform vec3 lightPositionPoint6;
uniform vec3 lightPositionPoint7;
uniform vec3 lightPositionPoint8;
vec3 lightPointPosition[numberOfPointLights];


vec3 calculateSpotLight(vec3 normal);
vec3 calculatePointLight(vec3 normal);

void main()
{    

	vec3 norm = normalize(fragNormal);

	vec3 result = calculatePointLight(norm);

	result += calculateSpotLight(norm);

	color = vec4(result, 1.0);

	
}


vec3 calculateSpotLight(vec3 normal){
	vec3 textureColor = texture(texture_diffuse1, fragUV).rgb;
	vec3 lightDirection = normalize(lightPositionSpot - fragPos);

	float angle = max(dot(lightDirection, normal), 0);
	vec3 diffuse = angle * lightColor * textureColor;

	float ambientStrength = 0.01f;
    float specularStrength = 0.5f;
	float constant = 1.0f;
	float linear = 0.09;
	float quadratic = 0.032;

	float theta = dot(lightDirection, normalize(-viewPosition));

	if(theta > cutOff){
		float angle = max(dot(lightDirection, normal), 0);
		vec3 diffuse = angle * lightColor * textureColor;

		// calculate attentuation
		float distanceFromLight = length(lightPositionSpot - fragPos);
		float attenuation = 1.0f / (constant + linear * distanceFromLight + quadratic * (distanceFromLight * distanceFromLight));

		vec3 viewDir = normalize(viewPosition - fragPos);
		vec3 reflectDir = reflect(-lightDirection, normal);

		float spec = pow(max(dot(viewDir, reflectDir), 0.0), 32);
		vec3 specular = specularStrength * spec * lightColor;

		// now multiply with attentuation
		diffuse *= attenuation;
		specular *= attenuation;

		vec3 ambient = ambientStrength * lightColor  * textureColor;
		vec3 result = (ambient + diffuse + specular) * objectColor;
		return result;
	}else{
		return vec3(ambientStrength * lightColor  * textureColor);
	}
}
vec3 calculatePointLight(vec3 normal)
{
	lightPointPosition[0] = lightPositionPoint1;
	lightPointPosition[1] = lightPositionPoint2;
	lightPointPosition[2] = lightPositionPoint3;
	lightPointPosition[3] = lightPositionPoint4;
	lightPointPosition[4] = lightPositionPoint5;
	lightPointPosition[5] = lightPositionPoint6;
	lightPointPosition[6] = lightPositionPoint7;
	lightPointPosition[7] = lightPositionPoint8;	

	float ambientStrength = 0.01f;
    float specularStrength = 0.75f;
	float constant = 1.0f;
	float linear = 0.09;
	float quadratic = 0.032;

	vec3 textureColor = texture(texture_diffuse1, fragUV).rgb;
	vec3 viewDir = normalize(viewPosition - fragPos);
	vec3 lightDirection;
	float angle;
	vec3 diffuse;
	float distanceFromLight; 
	float attenuation;
	vec3 reflectDir; 
	float spec; 
	vec3 specular; 
	
	vec3 ambient = ambientStrength * lightColor  * textureColor;
	vec3 result = ambient;

	for (int index = 0; index < numberOfPointLights; index++)
	{
		lightDirection = normalize(lightPointPosition[index] - fragPos);
		angle = max(dot(lightDirection, normal), 0);
		diffuse = angle * lightColor * textureColor;

		// calculate attentuation
		distanceFromLight = length(lightPointPosition[index] - fragPos);
		attenuation = 1.0f / (constant + linear * distanceFromLight + quadratic * (distanceFromLight * distanceFromLight));

		reflectDir = reflect(-lightDirection, normal);

		spec = pow(max(dot(viewDir, reflectDir), 0.0), 32);
		specular = specularStrength * spec * lightColor;

		// now multiply with attentuation
		diffuse *= attenuation;
		specular *= attenuation;

		result += (diffuse + specular) * objectColor;
	}
	return result;
}
